Enum argon2_kdf::Argon2Error
source · pub enum Argon2Error {
InvalidParameter(&'static str),
InvalidHash(&'static str),
CLibError(String),
}Expand description
Errors that may occur when using this crate
Variants
InvalidParameter(&'static str)
Indicates that the user of a type or function has specified an invalid parameter or set of parameters
InvalidHash(&'static str)
Indicates that a provided hash was expected to be valid, but is invalid. This normally occurs when a hash is improperly formatted.
CLibError(String)
An error that is unhandled by the crate, but is recognized by the C argon2 library