Struct argon2_kdf::Hash

source ·
pub struct Hash { /* private fields */ }
A container for an Argon2 hash, the corresponding salt, and the parameters used for hashing

Implementations

source

impl Hash

source

pub fn as_bytes(&self) -> &[u8]

Returns a reference to a byte slice of the computed hash/key.

source

pub fn salt(&self) -> &[u8]

Returns a reference to a byte slice of the salt used to generate the hash.

source

pub fn verify(&self, password: &[u8]) -> bool

Checks if the hash matches the provided password.

Because verification requires re-hashing the password, this is an expensive operation. For some appliations, it might make sense to move this operation to a separate thread using std::thread or something like the Rayon crate to avoid blocking main threads.

source

pub fn verify_with_secret(&self, password: &[u8], secret: Secret<'_>) -> bool

Checks if the hash matches the provided password using the provided secret.

Because verification requires re-hashing the password, this is an expensive operation. For some appliations, it might make sense to move this operation to a separate thread using std::thread or something like the Rayon crate to avoid blocking main threads.