Struct argon2_kdf::Hash
source · pub struct Hash { /* private fields */ }Expand description
A container for an Argon2 hash, the corresponding salt, and the parameters used for hashing
Implementations
sourceimpl Hash
impl Hash
sourcepub fn as_bytes<'a>(&'a self) -> &'a [u8] ⓘ
pub fn as_bytes<'a>(&'a self) -> &'a [u8] ⓘ
Returns a reference to a byte slice of the computed hash/key.
sourcepub fn salt<'a>(&'a self) -> &'a [u8] ⓘ
pub fn salt<'a>(&'a self) -> &'a [u8] ⓘ
Returns a reference to a byte slice of the salt used to generate the hash.
sourcepub fn verify(&self, password: &str) -> bool
pub fn verify(&self, password: &str) -> bool
Checks if the hash matches the provided password.
Because verification requires re-hashing the password, this is an expensive operation.
For some appliations, it might make sense to move this operation to a separate thread
using std::thread or something like
the Rayon crate to avoid blocking main threads.
sourcepub fn verify_with_secret(&self, password: &str, secret: Secret<'_>) -> bool
pub fn verify_with_secret(&self, password: &str, secret: Secret<'_>) -> bool
Checks if the hash matches the provided password using the provided secret.
Because verification requires re-hashing the password, this is an expensive operation.
For some appliations, it might make sense to move this operation to a separate thread
using std::thread or something like
the Rayon crate to avoid blocking main threads.